@@ -1,39 +1,41 @@
-#' @import Matrix ggplot2 ggraph patchwork
+#' @import ggplot2 ggraph ggnetwork
 #'
 #' @importFrom Rcpp evalCpp sourceCpp
 #' @importFrom RcppArmadillo armadillo_version
 #' @importFrom RcppParallel RcppParallelLibs
 #' @importFrom stats coef predict
-#' @importFrom utils methods
 NULL
 
 utils::globalVariables(
   c(
+    "Actual",
+    "Category",
+    "celltype",
+    "cluster",
     "count",
-    "x",
-    "y",
-    "xend",
-    "yend",
-    "regulator",
-    "target",
-    "weight",
-    "i",
-    "Interaction",
-    "name",
+    "curvetype",
     "degree",
     "edges",
-    "cluster",
-    "curvetype",
-    "weight_new",
-    "celltype",
     "from",
+    "i",
     "id",
+    "Interaction",
     "label_genes",
-    "targets_num",
-    "to",
+    "Metric",
+    "name",
     "P_value",
     "P_value_contrary",
-    "v",
-    "v_new"
+    "Predicted",
+    "regulator",
+    "targets_num",
+    "target",
+    "to",
+    "type",
+    "Value",
+    "weight",
+    "x",
+    "xend",
+    "y",
+    "yend"
   )
 )
