@@ -2,28 +2,24 @@
 #'
 #' @useDynLib inferCSN
 #'
+#' @md
 #' @param object The input data for *`inferCSN`*.
 #' @param penalty The type of regularization, default is *`L0`*.
 #' This can take either one of the following choices: *`L0`*, *`L0L1`*, and *`L0L2`*.
 #' For high-dimensional and sparse data, *`L0L2`* is more effective.
-#' @param algorithm The type of algorithm used to minimize the objective function, default is *`CD`*.
-#' Currently *`CD`* and *`CDPSI`* are supported.
-#' The *`CDPSI`* algorithm may yield better results, but it also increases running time.
 #' @param cross_validation Logical value, default is *`FALSE`*, whether to use cross-validation.
-#' @param n_folds The number of folds for cross-validation, default is *`10`*.
+#' @param n_folds The number of folds for cross-validation, default is *`5`*.
 #' @param seed The random seed for cross-validation, default is *`1`*.
-#' @param percent_samples The percent of all samples used for \code{\link{sparse_regression}}, default is *`1`*.
-#' @param r_threshold Threshold of \eqn{R^2} or correlation coefficient, default is *`0`*.
+#' @param subsampling_method The method to use for subsampling. Options are "sample", "pseudobulk" or "meta_cells".
+#' @param subsampling_ratio The percent of all samples used for \code{\link{fit_srm}}, default is *`1`*.
+#' @param r_squared_threshold Threshold of \eqn{R^2} coefficient, default is *`0`*.
 #' @param regulators The regulator genes for which to infer the regulatory network.
 #' @param targets The target genes for which to infer the regulatory network.
-#' @param regulators_num The number of non-zore coefficients, this value will affect the final performance.
-#' The maximum support size at which to terminate the regularization path.
 #' Recommend setting this to a small fraction of min(n,p) (e.g. 0.05 * min(n,p)) as L0 regularization typically selects a small portion of non-zeros.
 #' @param cores The number of cores to use for parallelization with \code{\link[foreach]{foreach}}, default is *`1`*.
 #' @param verbose Logical value, default is *`TRUE`*, whether to print progress messages.
 #' @param ... Parameters for other methods.
 #'
-#' @md
 #' @docType methods
 #' @rdname inferCSN
 #' @return A data table of regulator-target regulatory relationships
@@ -33,15 +29,16 @@ setGeneric(
   signature = c("object"),
   def = function(object,
                  penalty = "L0",
-                 algorithm = "CD",
                  cross_validation = FALSE,
                  seed = 1,
-                 n_folds = 10,
-                 percent_samples = 1,
-                 r_threshold = 0,
+                 n_folds = 5,
+                 subsampling_method = c(
+                   "sample", "meta_cells", "pseudobulk"
+                 ),
+                 subsampling_ratio = 1,
+                 r_squared_threshold = 0,
                  regulators = NULL,
                  targets = NULL,
-                 regulators_num = NULL,
                  cores = 1,
                  verbose = TRUE,
                  ...) {
@@ -60,13 +57,14 @@ setGeneric(
 #' network_table_1 <- inferCSN(
 #'   example_matrix
 #' )
-#' head(network_table_1)
 #'
 #' network_table_2 <- inferCSN(
 #'   example_matrix,
 #'   cores = 2
 #' )
 #'
+#' head(network_table_1)
+#'
 #' identical(
 #'   network_table_1,
 #'   network_table_2
@@ -82,72 +80,85 @@ setGeneric(
 #'   regulators = c("g1", "g2"),
 #'   targets = c("g3", "g0")
 #' )
-#' inferCSN(
+#'
+#' \dontrun{
+#' data("example_ground_truth")
+#' network_table_07 <- inferCSN(
 #'   example_matrix,
-#'   regulators = c("g1", "g0"),
-#'   targets = c("g2", "g3")
+#'   r_squared_threshold = 0.7
 #' )
-#' inferCSN(
-#'   example_matrix,
-#'   regulators = c("g1"),
-#'   targets = c("g2")
+#' calculate_metrics(
+#'   network_table_1,
+#'   example_ground_truth,
+#'   return_plot = TRUE
 #' )
+#' calculate_metrics(
+#'   network_table_07,
+#'   example_ground_truth,
+#'   return_plot = TRUE
+#' )
+#' }
 setMethod(
   f = "inferCSN",
   signature = signature(object = "matrix"),
   definition = function(object,
                         penalty = "L0",
-                        algorithm = "CD",
                         cross_validation = FALSE,
                         seed = 1,
-                        n_folds = 10,
-                        percent_samples = 1,
-                        r_threshold = 0,
+                        n_folds = 5,
+                        subsampling_method = c(
+                          "sample", "meta_cells", "pseudobulk"
+                        ),
+                        subsampling_ratio = 1,
+                        r_squared_threshold = 0,
                         regulators = NULL,
                         targets = NULL,
-                        regulators_num = NULL,
                         cores = 1,
                         verbose = TRUE,
                         ...) {
     log_message(
-      "Running for <", class(object)[1], ">.",
+      "Running for <dense matrix>.",
       verbose = verbose
     )
 
     .check_parameters(
       matrix = object,
       penalty = penalty,
-      algorithm = algorithm,
       cross_validation = cross_validation,
       seed = seed,
       n_folds = n_folds,
-      percent_samples = percent_samples,
-      r_threshold = r_threshold,
+      subsampling_method = subsampling_method,
+      subsampling_ratio = subsampling_ratio,
+      r_squared_threshold = r_squared_threshold,
       regulators = regulators,
       targets = targets,
-      regulators_num = regulators_num,
       verbose = verbose,
       cores = cores,
       ...
     )
 
-    if (!is.null(regulators)) {
-      regulators <- intersect(colnames(object), regulators)
-    } else {
-      regulators <- colnames(object)
-    }
-    if (!is.null(targets)) {
-      targets <- intersect(colnames(object), targets)
-    } else {
-      targets <- colnames(object)
-    }
-    if (is.null(regulators_num)) {
-      regulators_num <- (ncol(object) - 1)
-    }
+    object <- subsampling(
+      matrix = object,
+      subsampling_method = subsampling_method,
+      subsampling_ratio = subsampling_ratio,
+      seed = seed,
+      verbose = verbose,
+      ...
+    )
+
+    regulators <- intersect(
+      colnames(object),
+      regulators %ss% colnames(object)
+    )
+    targets <- intersect(
+      colnames(object),
+      targets %ss% colnames(object)
+    )
+
     names(targets) <- targets
     cores <- .cores_detect(cores, length(targets))
 
-    weight_list <- parallelize_fun(
+    network_table <- parallelize_fun(
       x = targets,
       fun = function(x) {
         single_network(
@@ -157,23 +168,24 @@ setMethod(
           cross_validation = cross_validation,
           seed = seed,
           penalty = penalty,
-          algorithm = algorithm,
           n_folds = n_folds,
-          percent_samples = percent_samples,
-          r_threshold = r_threshold,
-          regulators_num = regulators_num,
-          verbose = verbose
+          subsampling_ratio = subsampling_ratio,
+          r_squared_threshold = r_squared_threshold,
+          verbose = verbose,
+          ...
         )
       },
       cores = cores,
       verbose = verbose
+    ) |>
+      purrr::list_rbind() |>
+      network_format(abs_weight = FALSE)
+
+    log_message(
+      "Run done.",
+      message_type = "success",
+      verbose = verbose
     )
-    network_table <- purrr::list_rbind(weight_list)
-    network_table <- network_format(
-      network_table,
-      abs_weight = FALSE
-    )
-    log_message("Run done.", verbose = verbose)
 
     return(network_table)
   }
@@ -226,15 +238,16 @@ setMethod(
   signature = signature(object = "sparseMatrix"),
   definition = function(object,
                         penalty = "L0",
-                        algorithm = "CD",
                         cross_validation = FALSE,
                         seed = 1,
-                        n_folds = 10,
-                        percent_samples = 1,
-                        r_threshold = 0,
+                        n_folds = 5,
+                        subsampling_method = c(
+                          "sample", "meta_cells", "pseudobulk"
+                        ),
+                        subsampling_ratio = 1,
+                        r_squared_threshold = 0,
                         regulators = NULL,
                         targets = NULL,
-                        regulators_num = NULL,
                         cores = 1,
                         verbose = TRUE,
                         ...) {
@@ -246,37 +259,41 @@ setMethod(
     .check_parameters(
       matrix = object,
       penalty = penalty,
-      algorithm = algorithm,
       cross_validation = cross_validation,
       seed = seed,
       n_folds = n_folds,
-      percent_samples = percent_samples,
-      r_threshold = r_threshold,
+      subsampling_method = subsampling_method,
+      subsampling_ratio = subsampling_ratio,
+      r_squared_threshold = r_squared_threshold,
       regulators = regulators,
       targets = targets,
-      regulators_num = regulators_num,
       verbose = verbose,
       cores = cores,
       ...
     )
 
-    if (!is.null(regulators)) {
-      regulators <- intersect(colnames(object), regulators)
-    } else {
-      regulators <- colnames(object)
-    }
-    if (!is.null(targets)) {
-      targets <- intersect(colnames(object), targets)
-    } else {
-      targets <- colnames(object)
-    }
-    if (is.null(regulators_num)) {
-      regulators_num <- (ncol(object) - 1)
-    }
+    object <- subsampling(
+      matrix = object,
+      subsampling_method = subsampling_method,
+      subsampling_ratio = subsampling_ratio,
+      seed = seed,
+      verbose = verbose,
+      ...
+    )
+
+    regulators <- intersect(
+      colnames(object),
+      regulators %ss% colnames(object)
+    )
+    targets <- intersect(
+      colnames(object),
+      targets %ss% colnames(object)
+    )
+
     names(targets) <- targets
     cores <- .cores_detect(cores, length(targets))
 
-    weight_list <- parallelize_fun(
+    network_table <- parallelize_fun(
       x = targets,
       fun = function(x) {
         single_network(
@@ -286,23 +303,23 @@ setMethod(
           cross_validation = cross_validation,
           seed = seed,
           penalty = penalty,
-          algorithm = algorithm,
           n_folds = n_folds,
-          percent_samples = percent_samples,
-          r_threshold = r_threshold,
-          regulators_num = regulators_num,
-          verbose = verbose
+          r_squared_threshold = r_squared_threshold,
+          verbose = verbose,
+          ...
         )
       },
       cores = cores,
       verbose = verbose
+    ) |>
+      purrr::list_rbind() |>
+      network_format(abs_weight = FALSE)
+
+    log_message(
+      "Run done.",
+      message_type = "success",
+      verbose = verbose
     )
-    network_table <- purrr::list_rbind(weight_list)
-    network_table <- network_format(
-      network_table,
-      abs_weight = FALSE
-    )
-    log_message("Run done.", verbose = verbose)
 
     return(network_table)
   }
@@ -315,20 +332,21 @@ setMethod(
   signature = signature(object = "data.frame"),
   definition = function(object,
                         penalty = "L0",
-                        algorithm = "CD",
                         cross_validation = FALSE,
                         seed = 1,
-                        n_folds = 10,
-                        percent_samples = 1,
-                        r_threshold = 0,
+                        n_folds = 5,
+                        subsampling_method = c(
+                          "sample", "meta_cells", "pseudobulk"
+                        ),
+                        subsampling_ratio = 1,
+                        r_squared_threshold = 0,
                         regulators = NULL,
                         targets = NULL,
-                        regulators_num = NULL,
                         cores = 1,
                         verbose = TRUE,
                         ...) {
     log_message(
-      "Converting class type of input data from <data.frame> to <matrix>.",
+      "convert the class type of the input data from <data.frame> to <matrix>.",
       message_type = "warning",
       verbose = verbose
     )
@@ -336,15 +354,14 @@ setMethod(
     inferCSN(
       object = as_matrix(object),
       penalty = penalty,
-      algorithm = algorithm,
       cross_validation = cross_validation,
       seed = seed,
       n_folds = n_folds,
-      percent_samples = percent_samples,
-      r_threshold = r_threshold,
+      subsampling_method = subsampling_method,
+      subsampling_ratio = subsampling_ratio,
+      r_squared_threshold = r_squared_threshold,
       regulators = regulators,
       targets = targets,
-      regulators_num = regulators_num,
       verbose = verbose,
       cores = cores,
       ...
